"""
Currex - A Python library for currency conversion and arithmetic
"""

from .currency import Currency
from .factory import (
    USD,
    EUR,
    GBP,
    JPY,
    CHF,
    AUD,
    CAD,
    CNY,
    HKD,
    NZD,
    SEK,
    KRW,
    SGD,
    NOK,
    MXN,
    INR,
    RUB,
    ZAR,
    TRY,
    BRL,
    TWD,
    DKK,
    PLN,
    THB,
    IDR,
    CZK,
    AED,
    AFN,
    AOA,
    ARS,
    BAM,
    BGN,
    BYN,
    CDF,
    CLP,
    COP,
    CUP,
    GEL,
    GHS,
    GNF,
    HUF,
    ILS,
    ISK,
    KZT,
    MDL,
    MGA,
    MRU,
    MZN,
    NIO,
    PEN,
    PHP,
    PKR,
    RON,
    RSD,
    SDG,
    SRD,
    STN,
    TJS,
    TMT,
    UAH,
    UGX,
    UYU,
    UZS,
    VES,
    XAF,
    XOF,
    ZMW,
)
from .exchange import ExchangeRateAPI

__version__ = "0.1.1"
__all__ = [
    "Currency",
    "ExchangeRateAPI",
    "USD",
    "EUR",
    "GBP",
    "JPY",
    "CHF",
    "AUD",
    "CAD",
    "CNY",
    "HKD",
    "NZD",
    "SEK",
    "KRW",
    "SGD",
    "NOK",
    "MXN",
    "INR",
    "RUB",
    "ZAR",
    "TRY",
    "BRL",
    "TWD",
    "DKK",
    "PLN",
    "THB",
    "IDR",
    "CZK",
    "AED",
    "AFN",
    "AOA",
    "ARS",
    "BAM",
    "BGN",
    "BYN",
    "CDF",
    "CLP",
    "COP",
    "CUP",
    "GEL",
    "GHS",
    "GNF",
    "HUF",
    "ILS",
    "ISK",
    "KZT",
    "MDL",
    "MGA",
    "MRU",
    "MZN",
    "NIO",
    "PEN",
    "PHP",
    "PKR",
    "RON",
    "RSD",
    "SDG",
    "SRD",
    "STN",
    "TJS",
    "TMT",
    "UAH",
    "UGX",
    "UYU",
    "UZS",
    "VES",
    "XAF",
    "XOF",
    "ZMW",
]
